from tackle import tackle
from tackle.cli import main


# def test_lazy_error(change_curdir_fixtures):
#     main(["lazy.yaml", "-p"])
#
#
# def test_lazy_error_main(change_curdir_fixtures):
#     output = tackle("lazy.yaml")
#     assert output
#
#
# def test_if_issue(change_curdir_fixtures):
#     main(["ifdefined.yaml"])

    # output = tackle("ifdefined.yaml")
    # assert output
