# from tackle.main import tackle
#
# # TODO: https://github.com/robcxyz/tackle-box/issues/46
#
# def test_provider_pyinquirer_input_hook(change_dir, mocker):
#     # mocker.patch(
#     #     # 'tackle.providers.prompts.hooks.input.prompt',
#     #     'tackle.main.update_source',
#     #     return_value={"tmp": True},
#     #     autospec=True
#     # )
#     o = tackle()
#     assert o['this']
